"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DebugServer = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Copyright (c) 2017-present, Facebook, Inc.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * All rights reserved.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * This source code is licensed under the BSD-style license found in the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * LICENSE file in the root directory of this source tree. An additional grant
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * of patent rights can be found in the PATENTS file in the same directory.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */

var _BreakpointCollection = require("./BreakpointCollection.js");

var _invariant = require("../invariant.js");

var _invariant2 = _interopRequireDefault(_invariant);

var _DebugChannel = require("../DebugChannel.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DebugServer = exports.DebugServer = function () {
  function DebugServer(channel) {
    _classCallCheck(this, DebugServer);

    this.breakpoints = new _BreakpointCollection.BreakpointCollection();
    this.previousExecutedLine = 0;
    this.previousExecutedCol = 0;
    this.channel = channel;
    this.waitForRun(function (line) {
      return line === "Run";
    });
  }
  // the collection of breakpoints

  // the channel to communicate with the adapter


  _createClass(DebugServer, [{
    key: "waitForRun",


    /* Block until adapter says to run
    /* runCondition: a function that determines whether the adapter has told
    /* Prepack to continue running
    */
    value: function waitForRun(runCondition) {
      var blocking = true;
      var line = "";
      while (blocking) {
        line = this.channel.readIn().toString();
        if (runCondition(line)) {
          //The adapter gave the command to continue running
          //The caller (or someone else later) needs to send a response
          //to the adapter
          //We cannot pass in the response too because it may not be ready
          //immediately after Prepack unblocks
          blocking = false;
        } else {
          //The adapter gave another command so Prepack still blocks
          //but can read in other commands and respond to them
          this.executeCommand(line);
        }
      }
    }

    // Checking if the debugger needs to take any action on reaching this ast node

  }, {
    key: "checkForActions",
    value: function checkForActions(ast) {
      this.checkForBreakpoint(ast);

      // last step: set the current location as the previously executed line
      if (ast.loc && ast.loc.source !== null) {
        this.previousExecutedFile = ast.loc.source;
        this.previousExecutedLine = ast.loc.start.line;
        this.previousExecutedCol = ast.loc.start.column;
      }
    }
  }, {
    key: "proceedBreakpoint",
    value: function proceedBreakpoint(filePath, lineNum, colNum) {
      var breakpoint = this.breakpoints.getBreakpoint(filePath, lineNum, colNum);
      if (breakpoint && breakpoint.enabled) {
        // checking if this is the same file and line we stopped at last time
        // if so, we should skip it this time
        // Note: for the case when the debugger is supposed to stop on the same
        // breakpoint consecutively (e.g. the statement is in a loop), some other
        // ast node (e.g. block, loop) must have been checked in between so
        // previousExecutedFile and previousExecutedLine will have changed
        if (filePath === this.previousExecutedFile && lineNum === this.previousExecutedLine && colNum === this.previousExecutedCol) {
          return false;
        }
        return true;
      }
      return false;
    }
  }, {
    key: "checkForBreakpoint",
    value: function checkForBreakpoint(ast) {
      if (ast.loc && ast.loc.source) {
        var location = ast.loc;
        var filePath = location.source;
        if (filePath === null) return;
        var lineNum = location.start.line;
        var colNum = location.start.column;

        // Check whether there is a breakpoint we need to stop on here
        if (!this.proceedBreakpoint(filePath, lineNum, colNum)) return;

        // Tell the adapter that Prepack has stopped on this breakpoint
        this.channel.writeOut("breakpoint stopped " + lineNum + ":" + colNum);

        // Wait for the adapter to tell us to run again
        this.waitForRun(function (line) {
          return line === "proceed";
        });
      }
    }
  }, {
    key: "executeCommand",
    value: function executeCommand(command) {
      if (command.length === 0) {
        return;
      }
      var parts = command.split(" ");
      if (parts[0] === "breakpoint") {
        this.executeBreakpointCommand(this._parseBreakpointArguments(parts));
      }
    }
  }, {
    key: "executeBreakpointCommand",
    value: function executeBreakpointCommand(args) {
      if (args.kind === "add") {
        this.breakpoints.addBreakpoint(args.filePath, args.lineNum, args.columnNum);
        this.channel.writeOut("added breakpoint " + args.filePath + " " + args.lineNum + " " + args.columnNum);
      } else if (args.kind === "remove") {
        this.breakpoints.removeBreakpoint(args.filePath, args.lineNum, args.columnNum);
        this.channel.writeOut("removed breakpoint " + args.filePath + " " + args.lineNum + " " + args.columnNum);
      } else if (args.kind === "enable") {
        this.breakpoints.enableBreakpoint(args.filePath, args.lineNum, args.columnNum);
        this.channel.writeOut("enabled breakpoint " + args.filePath + " " + args.lineNum + " " + args.columnNum);
      } else if (args.kind === "disable") {
        this.breakpoints.disableBreakpoint(args.filePath, args.lineNum, args.columnNum);
        this.channel.writeOut("disabled breakpoint " + args.filePath + " " + args.lineNum + " " + args.columnNum);
      }
    }
  }, {
    key: "_parseBreakpointArguments",
    value: function _parseBreakpointArguments(parts) {
      (0, _invariant2.default)(parts[0] === "breakpoint");
      var kind = parts[1];
      var filePath = parts[2];

      var lineNum = parseInt(parts[3], 10);
      (0, _invariant2.default)(!isNaN(lineNum));
      var columnNum = 0;
      if (parts.length === 5) {
        columnNum = parseInt(parts[4], 10);
        (0, _invariant2.default)(!isNaN(columnNum));
      }

      var result = {
        kind: kind,
        filePath: filePath,
        lineNum: lineNum,
        columnNum: columnNum
      };

      return result;
    }
  }, {
    key: "shutdown",
    value: function shutdown() {
      //let the adapter know Prepack is done running
      this.channel.writeOut("Finished");
    }
  }]);

  return DebugServer;
}();
//# sourceMappingURL=Debugger.js.map